<?php
namespace MainWP\Extensions\SSL_Monitor;

class MainWP_SSL_Monitor_Admin {
	public $version           = '2.0';
	public static $sort_field = '';

	/**
	 * Static variable to hold the single instance of the class.
	 *
	 * @static
	 *
	 * @var mixed Default null
	 */
	static $instance = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_SSL_Monitor_Admin
	 */
	static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * Runs each time the class is called.
	 */
	public function __construct() {
		add_action( 'init', array( &$this, 'init' ) );
		add_action( 'init', array( &$this, 'localization' ) );
		add_filter( 'mainwp_specific_action_logs', array( &$this, 'hook_mainwp_specific_action_logs' ), 10, 2 );
	}

	/**
	 * Initiate Hooks
	 *
	 * Initiates hooks for the SSL Monitor extension.
	 */
	public function init() {
		add_filter( 'plugin_row_meta', array( &$this, 'plugin_row_meta' ), 10, 2 );
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		add_action( 'mainwp_help_sidebar_content', array( $this, 'mainwp_help_content' ) );
		add_filter( 'mainwp_header_actions_right', array( $this, 'screen_options' ), 10, 2 );

		MainWP_SSL_Monitor_DB::get_instance()->init();
		MainWP_SSL_Monitor_Hooks::get_instance()->init();
		MainWP_SSL_Monitor_Schedule::get_instance()->init();

		$this->handle_sites_screen_settings();

		if ( self::is_managesites_page() ) {
			$current_site_id = intval( $_GET['id'] );
			self::handle_individual_settings_post( $current_site_id );
		} else {
			self::handle_general_settings_post();
		}
	}


	/**
	 * Localization
	 *
	 * Sets the localization domain.
	 */
	public function localization() {
		load_plugin_textdomain( 'mainwp-ssl-monitor-extension', false, MAINWP_SSL_MONITOR_PLUGIN_DIR . '/languages/' );
	}

	/**
	 * Sites Page Check
	 *
	 * Checks if the current page is individual site SSL Monitor page.
	 *
	 * @return bool True if correct, false if not.
	 */
	public static function is_managesites_page() {
		if ( isset( $_GET['page'] ) && ( 'ManageSitesSSLMonitor' == $_GET['page'] ) ) {
			return true;
		}
			return false;
	}


	/**
	 * Plugin Row Meta
	 *
	 * Displays the meta in the plugin row on the WP > Plugins > Installed Plugins page.
	 *
	 * @param array  $plugin_meta Plugin meta data.
	 * @param string $plugin_file Plugin file.
	 *
	 * @return array  $plugin_meta Plugin meta data.
	 */
	public function plugin_row_meta( $plugin_meta, $plugin_file ) {
		if ( MAINWP_SSL_MONITOR_PLUGIN_SLUG != $plugin_file ) {
			return $plugin_meta;
		}

		$slug     = basename( $plugin_file, '.php' );
		$api_data = get_option( $slug . '_APIManAdder' );

		if ( ! is_array( $api_data ) || ! isset( $api_data['activated_key'] ) || $api_data['activated_key'] != 'Activated' || ! isset( $api_data['api_key'] ) || empty( $api_data['api_key'] ) ) {
			return $plugin_meta;
		}

		$plugin_meta[] = '<a href="?do=checkUpgrade" title="Check for updates.">Check for updates now</a>';

		return $plugin_meta;
	}

	/**
	 * Admin Init
	 *
	 * Initiates admin hooks.
	 */
	public function admin_init() {
		if ( isset( $_GET['page'] ) && ( 'Extensions-Mainwp-Ssl-Monitor-Extension' == $_GET['page'] || 'mainwp_tab' == $_GET['page'] || 'managesites' == $_GET['page'] || 'ManageSitesSSLMonitor' == $_GET['page'] ) ) {
			wp_enqueue_style( 'mainwp-ssl-monitor-extension', MAINWP_SSL_MONITOR_PLUGIN_URL . 'css/mainwp-ssl-monitor-extension.css', array(), $this->version );
			wp_enqueue_script( 'mainwp-ssl-monitor-extension', MAINWP_SSL_MONITOR_PLUGIN_URL . 'js/mainwp-ssl-monitor-extension.js', array( 'jquery' ), $this->version );
			wp_localize_script(
				'mainwp-ssl-monitor-extension',
				'mainwpSSLMonitor',
				array(
					'nonce' => wp_create_nonce( 'mwp_ssl_monitor_nonce' ),
				)
			);
		}
		MainWP_SSL_Monitor_Dashboard::get_instance()->admin_init();
	}

	/**
	 * Render tabs
	 *
	 * Renders the extension page tabs.
	 */
	public static function render_extension_page() {

		$current_site_id = null;
		$website         = null;

		if ( isset( $_GET['id'] ) && ! empty( $_GET['id'] ) ) {
			$current_site_id = $_GET['id'];
			$dbwebsites      = self::get_db_sites( array( $current_site_id ) );
			if ( is_array( $dbwebsites ) && ! empty( $dbwebsites ) ) {
				$website = current( $dbwebsites );
			}
		}

		if ( $current_site_id ) {
			$error = '';
			if ( empty( $website ) || empty( $website->id ) ) {
				$error = __( 'Undefined site id. Please, try again.', 'mainwp-ssl-monitor-extension' );
			}

			do_action( 'mainwp_pageheader_sites', 'SSL_Monitor' );

			if ( ! empty( $error ) ) {
				echo '<div class="ui segment">';
				echo '<div class="ui yellow message">' . $error . '</div>';
				echo '</div>';
			} else {

				self::render_extension_page_site( $website );
			}

			do_action( 'mainwp_pagefooter_sites', 'SSL_Monitor' );
		} else {
			self::render_extension_page_general();
		}
	}

	/**
	 * Render extension page
	 *
	 * Renders the main extension page.
	 */
	public static function render_extension_page_general() {
		$curent_tab = 'dashboard';
		if ( isset( $_GET['tab'] ) ) {
			if ( 'settings' == $_GET['tab'] ) {
					$curent_tab = 'settings';
			}
		} elseif ( isset( $_POST['mwp_ssl_monitor_setting_submit'] ) ) {
			$curent_tab = 'settings';
		}
		?>
		<div class="ui labeled icon inverted menu mainwp-sub-submenu" id="mainwp-ssl-monitor-menu">
			<a href="admin.php?page=Extensions-Mainwp-Ssl-Monitor-Extension&tab=dashboard" class="item <?php echo ( $curent_tab == 'dashboard' ? 'active' : '' ); ?>"><i class="tasks icon"></i> <?php _e( 'Dashboard', 'mainwp-ssl-monitor-extension' ); ?></a>
			<a href="admin.php?page=Extensions-Mainwp-Ssl-Monitor-Extension&tab=settings" class="item <?php echo ( $curent_tab == 'settings' ? 'active' : '' ); ?>"><i class="cog icon"></i> <?php _e( 'Settings', 'mainwp-ssl-monitor-extension' ); ?></a>
		</div>
		<?php if ( $curent_tab == 'dashboard' || $curent_tab == '' ) : ?>
			<?php MainWP_SSL_Monitor_Dashboard::render_actions_bar(); ?>
			<div class="ui segment">
				<?php MainWP_SSL_Monitor_Dashboard::gen_dashboard_tab(); ?>
			</div>
		<?php endif; ?>
		<?php if ( $curent_tab == 'settings' ) : ?>
			<div class="ui segment">
				<?php self::render_settings(); ?>
			</div>
		<?php endif; ?>
		<?php
	}

	/**
	 * Render settings
	 *
	 * Renders the extension settings page.
	 *
	 * @param int $individual Individual or General settings.
	 */
	public static function render_settings( $individual = false ) {

		$current_site_id = 0;
		if ( self::is_managesites_page() ) {
			$current_site_id = $_GET['id'];
		}

		$use_schedule           = 0;
		$frequency              = 0;
		$notification_threshold = '';

		if ( $current_site_id ) {
			$site_settings = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $current_site_id );
		} else {
			$use_schedule           = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'use_schedule' );
			$frequency              = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'frequency' );
			$notification_threshold = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'notification_threshold' );
		}

		$overwrite = 0;

		if ( $current_site_id && $site_settings ) {
			$settings = isset( $site_settings->settings ) ? $site_settings->settings : '';
			if ( is_array( $settings ) ) {
				$use_schedule           = isset( $settings['use_schedule'] ) ? $settings['use_schedule'] : '';
				$frequency              = isset( $settings['frequency'] ) ? $settings['frequency'] : '';
				$notification_threshold = isset( $settings['notification_threshold'] ) ? $settings['notification_threshold'] : '';
			}
			$overwrite = $site_settings->overwrite;
		}

		$frequency              = empty( $frequency ) ? 86400 : $frequency;
		$notification_threshold = empty( $notification_threshold ) ? 7 : $notification_threshold;

		$auto_check_frequency = MainWP_SSL_Monitor_Utility::get_recheck_intervals();
		?>
		<?php if ( MainWP_SSL_Monitor_Utility::show_mainwp_message( 'mainwp-ssl-monitor-settings-info-message' ) ) : ?>
			<div class="ui info message">
				<i class="close icon mainwp-notice-dismiss" notice-id="mainwp-ssl-monitor-settings-info-message"></i>
				<?php printf( __( 'Manage the SSL Monitor settings. For detailed information, review %1$shelp documentation%2$s.', 'mainwp-ssl-monitor-extension' ), '<a href="https://kb.mainwp.com/docs/mainwp-ssl-monitor-extension/" target="_blank">', '</a>' ); ?>
			</div>
		<?php endif; ?>
		<form id="mainwp-ssl-monitor-settings-form" method="post" action="<?php echo ( $individual ? 'admin.php?page=ManageSitesSSLMonitor&id=' . $current_site_id . '&tab=settings' : 'admin.php?page=Extensions-Mainwp-Ssl-Monitor-Extension&tab=settings' ); ?>" class="ui form">
			<?php if ( $individual ) : ?>
			<h3 class="header"><?php echo __( 'Individual Site SSL  Monitor Settings', 'mainwp-ssl-monitor-extension' ); ?></h3>
			<div class="ui grid field">
				<label class="six wide column middle aligned"><?php _e( 'Overwrite general settings', 'mainwp-ssl-monitor-extension' ); ?></label>
				<div class="ten wide column ui toggle checkbox" data-tooltip="<?php esc_attr_e( 'Enable to overwrite the general settings for this child site.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="top left">
					<input type="checkbox" name="mainwp-ssl-monitor-overwrite-general-settings" id="mainwp-ssl-monitor-overwrite-general-settings" value="1" <?php echo ( 0 == $overwrite ? '' : 'checked="checked"' ); ?>><label></label>
				</div>
			</div>
			<?php endif; ?>
			<?php if ( $individual ) : ?>
			<div class="mainwp-ssl-monitor-overwrite-general-settings-toggle-area" <?php echo 0 == $overwrite ? 'style="display:none"' : ''; ?>>
			<?php endif; ?>
			<h3 class="header"><?php echo __( 'SSL Monitor Settings', 'mainwp-ssl-monitor-extension' ); ?></h3>
			<div class="ui grid field">
				<label class="six wide column middle aligned"><?php _e( 'Notifications threshold (days)', 'mainwp-ssl-monitor-extension' ); ?></label>
				<div class="ten wide column" data-tooltip="<?php esc_attr_e( 'Set the SSL Certificate expiration threshold for the email notification.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="top left">
					<input type="number" id="mainwp-ssl-monitor-notification-threshold" name="mainwp-ssl-monitor-notification-threshold" value="<?php echo $notification_threshold; ?>" />
				</div>
			</div>
			<h3 class="header"><?php echo __( 'Automated Checks', 'mainwp-ssl-monitor-extension' ); ?></h3>
			<div class="ui grid field">
				<label class="six wide column middle aligned"><?php _e( 'Automatically check SSL certificates', 'mainwp-ssl-monitor-extension' ); ?></label>
					<div class="ten wide column ui toggle checkbox" data-tooltip="<?php esc_attr_e( 'Enable if you want the extension to run SSL certificate checks automatically.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="top left">
					<input type="checkbox" name="mainwp-ssl-monitor-automatic-checks" id="mainwp-ssl-monitor-automatic-checks" value="1" <?php echo ( 1 == $use_schedule ? 'checked="checked"' : '' ); ?>><label></label>
				</div>
			</div>
			<div class="ui grid field" id="mainwp-ssl-monitor-automatic-checks-toggle-area" <?php echo $use_schedule == 1 ? '' : 'style="display:none"'; ?>>
				<label class="six wide column middle aligned"><?php _e( 'Automated SSL certificate checks frequency', 'mainwp-ssl-monitor-extension' ); ?></label>
					<div class="ten wide column" data-tooltip="<?php esc_attr_e( 'Select automated SSL certificate checks frequency.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="top left">
					<select id="mainwp-ssl-monitor-automatic-checks-frequency" name="mainwp-ssl-monitor-automatic-checks-frequency" class="ui dropdown">
						<?php foreach ( $auto_check_frequency as $key => $value ) : ?>
							<?php
							$_select = '';
							if ( $key == $frequency ) {
								$_select = ' selected ';
							}
							echo '<option value="' . $key . '" ' . $_select . '>' . $value . '</option>';
							?>
						<?php endforeach; ?>
					</select>
				</div>
			</div>
			<?php if ( $individual ) : ?>
			</div>
			<?php endif; ?>
			<input type="hidden" name="nonce" value="<?php echo wp_create_nonce( 'mwp_ssl_monitor_nonce' ); ?>">
			<input type="hidden" name="mwp_ssl_monitor_setting_submit" value="1">
			<div class="ui divider"></div>
			<input type="submit" name="submit" id="submit" class="ui big green button" value="<?php _e( 'Save Settings', 'mainwp-ssl-monitor-extension' ); ?>">
		</form>
		<?php
	}

	/**
	 * Render extension site page
	 *
	 * Renders the extension page for individual site.
	 */
	public static function render_extension_page_site( $website ) {
		$current_site_id = intval( $_GET['id'] );
		$curent_tab      = 'ssl-info';
		if ( isset( $_GET['tab'] ) && 'settings' == $_GET['tab'] ) {
			$curent_tab = 'settings';
		}
		?>
		<div class="ui labeled icon inverted menu mainwp-sub-submenu" id="mainwp-ssl-monitor-menu">
			<a href="admin.php?page=ManageSitesSSLMonitor&id=<?php echo intval( $current_site_id ); ?>&tab=ssl-info" class="item <?php echo 'ssl-info' == $curent_tab ? 'active' : ''; ?>"><i class="cog icon"></i><?php _e( 'SSL Certificate Info', 'mainwp-ssl-monitor-extension' ); ?></a>
			<a href="admin.php?page=ManageSitesSSLMonitor&id=<?php echo intval( $current_site_id ); ?>&tab=settings" class="item <?php echo 'settings' == $curent_tab ? 'active' : ''; ?>"><i class="cog icon"></i><?php _e( 'Settings', 'mainwp-ssl-monitor-extension' ); ?></a>
		</div>
		<?php if ( $curent_tab == 'ssl-info' || $curent_tab == '' ) : ?>
			<div class="ui segment">
				<?php self::render_ssl_profile( $current_site_id ); ?>
			</div>
		<?php endif; ?>
		<?php if ( $curent_tab == 'settings' ) : ?>
			<div class="ui segment">
				<?php self::render_settings( true ); ?>
			</div>
		<?php endif; ?>
		<?php
	}

	public static function render_ssl_profile( $website, $widget = false ) {
		$ssl_data = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $website );

		$dbwebsites = self::get_db_sites( array( $website ) );
		if ( is_array( $dbwebsites ) && ! empty( $dbwebsites ) ) {
			$site = current( $dbwebsites );
		}
		?>
		<?php if ( ! $widget ) : ?>
			<?php if ( MainWP_SSL_Monitor_Utility::starts_with( $site->url, 'https://' ) ) : ?>
				<?php if ( MainWP_SSL_Monitor_Utility::show_mainwp_message( 'mainwp-ssl-monitor-report-info-message' ) ) : ?>
			<div class="ui info message">
				<i class="close icon mainwp-notice-dismiss" notice-id="mainwp-ssl-monitor-report-info-message"></i>
				<div><?php echo __( 'Review the SSL Certificate info for the child site.', 'mainwp-ssl-monitor-extension' ); ?></div>
			</div>
		<?php endif; ?>
		<?php endif; ?>
			<?php if ( ! MainWP_SSL_Monitor_Utility::starts_with( $site->url, 'https://' ) ) : ?>
		<div class="ui info message"><?php esc_html_e( 'This seems to be a HTTP site. SSL Crtificate is not detected.', 'mainwp-ssl-monitor-extension' ); ?></div>
		<?php endif; ?>
		<?php endif; ?>
		<?php if ( ! empty( $ssl_data ) ) : ?>
		<table class="ui table">
			<thead>
				<tr>
					<th colspan="2"><?php _e( 'SSL Certificate Info', 'mainwp-ssl-monitor-extension' ); ?> (Last check: <?php echo MainWP_SSL_Monitor_Utility::format_timestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->last_check ) ); ?>)</th>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td><strong><?php _e( 'Name', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->name; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Issuer', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td>
						<?php echo '' != $ssl_data->issuer_o ? '(O) ' . $ssl_data->issuer_o . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->issuer_cn ? '(CN) ' . $ssl_data->issuer_cn . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->issuer_c ? '(C) ' . $ssl_data->issuer_c : ''; ?>
					</td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Subject', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td>
						<?php echo '' != $ssl_data->subject_c ? '(C) ' . $ssl_data->subject_c . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->subject_st ? '(ST) ' . $ssl_data->subject_st . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->subject_l ? '(L) ' . $ssl_data->subject_l . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->subject_o ? '(O) ' . $ssl_data->subject_o . '<br/>' : ''; ?>
						<?php echo '' != $ssl_data->subject_cn ? '(CN) ' . $ssl_data->subject_cn : ''; ?>
					</td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Valid From', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_from ) ); ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Valid To', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ); ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Subject Alternative Name', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->subject_alt_name; ?></td>
				</tr>
				<?php if ( ! $widget ) : ?>
				<tr>
					<td><strong><?php _e( 'Hash', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->hash; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Version', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->version; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Serial Number', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->serial_number; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Serial Number Hex', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->serial_number_hex; ?></td>
				</tr>
				
				<tr>
					<td><strong><?php _e( 'Signature Type SN', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->signature_type_sn; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Signature Type LN', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->signature_type_ln; ?></td>
				</tr>
				<tr>
					<td><strong><?php _e( 'Signature Type NID', 'mainwp-ssl-monitor-extension' ); ?></strong></td>
					<td><?php echo $ssl_data->signature_type_nid; ?></td>
				</tr>
				<?php endif; ?>
			</tbody>
		</table>
		<?php endif; ?>
		<?php
	}

	/**
	 * Render metabox
	 *
	 * Initiates the metabox.
	 */
	public static function render_metabox() {
		if ( ! isset( $_GET['page'] ) || 'managesites' == $_GET['page'] ) {
			self::site_overview_metabox();
		} else {
			self::overview_metabox();
		}
	}

	/**
	 * Global Metabox
	 *
	 * Renders the Overview page widget content.
	 */
	public static function overview_metabox() {
		$websites = MainWP_SSL_Monitor_Utility::get_websites();
		?>
		<div class="ui grid mainwp-widget-header">
			<div class="twelve wide column">
				<h3 class="ui header handle-drag">
					<?php echo __( 'SSL Monitor', 'mainwp-ssl-monitor-extension' ); ?>
					<div class="sub header"><?php echo __( 'Monitor SSL Certificates status.', 'mainwp-ssl-monitor-extension' ); ?></div>
				</h3>
			</div>
		</div>
		<div class="mainwp-scrolly-overflow">
		<table class="ui table" id="mainwp-ssl-monitor-table">
			<thead>
				<tr>
					<th class="collapsing no-sort"><?php echo __( '', 'mainwp-ssl-monitor-extension' ); ?></th>
					<th><?php echo __( 'Site', 'mainwp-ssl-monitor-extension' ); ?></th>
					<th class="collapsing"><?php echo __( 'Expiration Date', 'mainwp-ssl-monitor-extension' ); ?></th>
				</tr>
			</thead>
			<tbody>
				<?php
				foreach ( $websites as $website ) {
					$website_id = $website['id'];
					$ssl_data   = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $website_id );
					if ( ! empty( $ssl_data->valid_to ) ) {
						$expires = round( ( intval( $ssl_data->valid_to ) - time() ) / ( 60 * 60 * 24 ) );
					} else {
						$expires = 0;
					}
					$icon_code = '<i class="check large circle icon green"></i>';
					if ( 30 > $expires && $expires >= 1 ) {
						$icon_code = '<i class="warning large circle icon yellow"></i>';
					} elseif ( 1 > $expires ) {
						$icon_code = '<i class="times large circle icon red"></i>';
					}
					?>
					<?php if ( MainWP_SSL_Monitor_Utility::starts_with( $website['url'], 'https://' ) ) : ?>
					<tr>
						<td><span data-tooltip="Expires in <?php echo $expires; ?> day(s)." data-inverted="" data-position="right center"><?php echo $icon_code; ?></span></td>
						<td>
							<div><a href="admin.php?page=managesites&dashboard=<?php echo $website_id; ?>" data-tooltip="<?php esc_attr_e( 'Go to the site overview.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="right center" class="mainwp-site-name-link"><?php echo stripslashes( $website['name'] ); ?></a></div>
							<span class="ui small text"><a href="<?php echo $website['url']; ?>" target="_blank" class="open_site_url" data-tooltip="<?php esc_attr_e( 'Go to the website.', 'mainwp-ssl-monitor-extension' ); ?>" data-inverted="" data-position="right center"><?php echo $website['url']; ?></a></span>
						</td>
						<td><?php echo ! empty( $ssl_data->valid_to ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ) : 'N/A'; ?></td>
					</tr>
					<?php endif; ?>
					<?php
				}
				?>
			</tbody>
		</table>
		</div>
		
		<div class="ui two columns grid mainwp-widget-footer">
			<div class="column">
				<a href="admin.php?page=Extensions-Mainwp-Ssl-Monitor-Extension" class="ui basic green mini fluid button"><?php esc_html_e( 'SSL Monitor Dashboard', 'mainwp-ssl-monitor-extension' ); ?></a>
			</div>
			<?php do_action( 'mainwp_ssl_monitor_widget_bottom_buttons', false ); ?>
		</div>
		<script type="text/javascript">
		jQuery( document ).ready( function () {
			jQuery( '#mainwp-ssl-monitor-table' ).DataTable( {
				"stateSave": true,
				"stateDuration": 0,
				"scrollX": false,
				"lengthMenu": [ [10, 25, 50, 100, -1], [10, 25, 50, 100, "All"] ],
				"columnDefs": [ { "targets": 'no-sort', "orderable": false } ],
				"order": [ [ 1, "asc" ] ],
				"language": { "emptyTable": "No sites found." },
			} );
		} );
		</script>
		<?php

		if ( isset( $_GET['action'] ) && 'ssl_check' == $_GET['action'] && isset( $_GET['ssl_monitor_nonce'] ) && wp_verify_nonce( sanitize_text_field( $_GET['ssl_monitor_nonce'] ), 'ssl_monitor_nonce' ) ) {
			return MainWP_SSL_Monitor_Dashboard::mainwp_ssl_monitor_modal();
		}
	}

	/**
	 * Individual Metabox
	 *
	 * Renders the individual site Overview page widget content.
	 */
	public static function site_overview_metabox() {
		$site_id = isset( $_GET['dashboard'] ) ? $_GET['dashboard'] : 0;

		if ( empty( $site_id ) ) {
			return;
		}

		$dbwebsites = self::get_db_sites( array( $site_id ) );
		if ( is_array( $dbwebsites ) && ! empty( $dbwebsites ) ) {
			$website = current( $dbwebsites );
		}
		?>
		<div class="ui grid mainwp-widget-header">
			<div class="twelve wide column">
				<h3 class="ui header handle-drag">
					<?php echo __( 'SSL Monitor', 'mainwp-ssl-monitor-extension' ); ?>
					<div class="sub header"><?php echo __( 'SSL Monitor extension.', 'mainwp-ssl-monitor-extension' ); ?></div>
				</h3>
			</div>
		</div>
		<div class="mainwp-scrolly-overflow">
		<?php if ( MainWP_SSL_Monitor_Utility::starts_with( $website->url, 'https://' ) ) : ?>
			<?php self::render_ssl_profile( $site_id, true ); ?>
		<?php else : ?>
		<h2 class="ui icon header">
			<i class="warning circle icon"></i>
			<div class="content">
				<?php esc_html_e( 'SSL Certificate Not Detected!', 'mainwp-ssl-monitor-extension' ); ?>
				<div class="sub header"><?php esc_html_e( 'This seems to be a HTTP site. SSL Crtificate is not detected.', 'mainwp-ssl-monitor-extension' ); ?></div>
			</div>
		</h2>
		<?php endif; ?>
		</div>
		<div class="ui two columns grid mainwp-widget-footer">
			<div class="column">
				<a href="admin.php?page=Extensions-Mainwp-Ssl-Monitor-Extension" class="ui basic green mini fluid button"><?php esc_html_e( 'SSL Monitor Dashboard', 'mainwp-ssl-monitor-extension' ); ?></a>
			</div>
		</div>
		<?php
	}

	/**
	 * Individual Settigns Post
	 *
	 * Handles the Individual site save settings post request.
	 *
	 * @param int $websiteId Child site ID.
	 */
	public static function handle_individual_settings_post( $websiteId ) {
		if ( isset( $_POST['submit'] ) && $websiteId ) {
			self::handle_settings_post( $websiteId );
		}
	}

	/**
	 * General Settigns Post
	 *
	 * Handles the general save settings post request.
	 *
	 * @return mixed $save_output Save output.
	 */
	public static function handle_general_settings_post() {
		$save_output = self::handle_settings_post();
		return $save_output;
	}

	/**
	 * Check Security
	 *
	 * Verifies nonce for security reasons.
	 */
	public static function check_security() {
		if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( sanitize_text_field( $_POST['nonce'] ), 'mwp_ssl_monitor_nonce' ) ) {
			die( __( 'Invalid request. Please, try again.', 'mainwp-ssl-monitor-extension' ) );
		}
	}

	/**
	 * Settigns Post
	 *
	 * Handles the save settings post request.
	 *
	 * @param int $website_id Child site ID.
	 *
	 * @return mixed Save output.
	 */
	public static function handle_settings_post( $website_id = null ) {
		if ( isset( $_POST['mwp_ssl_monitor_setting_submit'] ) ) {
			self::check_security();
		}

		if ( isset( $_POST['mwp_ssl_monitor_setting_submit'] ) ) {
			$output = array();

			$use_schedule = '';
			if ( isset( $_POST['mainwp-ssl-monitor-automatic-checks'] ) ) {
				$use_schedule = intval( $_POST['mainwp-ssl-monitor-automatic-checks'] );
			}

			$frequency = 0;
			if ( isset( $_POST['mainwp-ssl-monitor-automatic-checks-frequency'] ) ) {
				$frequency = intval( $_POST['mainwp-ssl-monitor-automatic-checks-frequency'] );
			}

			$notification_threshold = 0;
			if ( isset( $_POST['mainwp-ssl-monitor-notification-threshold'] ) ) {
				$notification_threshold = intval( $_POST['mainwp-ssl-monitor-notification-threshold'] );
			}

			if ( ! $website_id ) {
				$old_interval = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'frequency' );

				MainWP_SSL_Monitor_Utility::get_instance()->set_option( 'use_schedule', $use_schedule );
				MainWP_SSL_Monitor_Utility::get_instance()->set_option( 'frequency', $frequency );
				MainWP_SSL_Monitor_Utility::get_instance()->set_option( 'notification_threshold', $notification_threshold );
				MainWP_SSL_Monitor_Schedule::get_instance()->init_jobs( $old_interval );
			} elseif ( $website_id ) {
				$overwrite = isset( $_POST['mainwp-ssl-monitor-overwrite-general-settings'] ) ? 1 : 0;

				$settings = array(
					'use_schedule'           => $use_schedule,
					'frequency'              => $frequency,
					'notification_threshold' => $notification_threshold,
				);

				$update              = array();
				$update['settings']  = wp_json_encode( $settings );
				$update['overwrite'] = $overwrite;
				$update['site_id']   = $website_id;

				$out = MainWP_SSL_Monitor_DB::get_instance()->update_ssl_monitor( $update );
			}
			return $output;
		}
		return false;
	}

	/**
	 * Get Websites
	 *
	 * Gets all child sites through the 'mainwp_getsites' filter.
	 *
	 * @param array $site_ids  Child sites IDs.
	 * @param array $group_ids Groups IDs.
	 *
	 * @return array Child sites array.
	 */
	public static function get_db_sites( $site_ids, $group_ids = array() ) {
		if ( ! is_array( $site_ids ) ) {
			$site_ids = array();
		}

		if ( ! is_array( $group_ids ) ) {
			$group_ids = array();
		}

		if ( ! empty( $site_ids ) || ! empty( $group_ids ) ) {
			global $mainWPSSLMonitorExtensionActivator;
			return apply_filters( 'mainwp_getdbsites', $mainWPSSLMonitorExtensionActivator->get_child_file(), $mainWPSSLMonitorExtensionActivator->get_child_key(), $site_ids, $group_ids );
		}
		return false;
	}

	/**
	 * Get Websites
	 *
	 * Gets all child sites through the 'mainwp_getsites' filter.
	 *
	 * @param array $site_ids  Child sites IDs.
	 * @param array $group_ids Groups IDs.
	 *
	 * @return array Child sites array.
	 */
	public static function get_websites( $site_ids = null, $group_ids = false ) {
		global $mainWPSSLMonitorExtensionActivator;
		return apply_filters( 'mainwp_getsites', $mainWPSSLMonitorExtensionActivator->get_child_file(), $mainWPSSLMonitorExtensionActivator->get_child_key(), $site_ids, $group_ids );
	}

	/**
	 * Widgets screen options.
	 *
	 * @param array $input Input.
	 *
	 * @return array $input Input.
	 */
	public function widgets_screen_options( $input ) {
		$input['advanced-ssl-monitor-widget'] = __( 'SSL Monitor', 'mainwp-ssl-monitor-extension' );
		return $input;
	}

	/**
	 * Method screen_options()
	 *
	 * Create Page Settings button.
	 *
	 * @param mixed $input Page Settings button HTML.
	 *
	 * @return mixed Page Settings button.
	 */
	public function screen_options( $input ) {
		if ( isset( $_GET['page'] ) && 'Extensions-Mainwp-Ssl-Monitor-Extension' == $_GET['page'] ) {
			if ( ! isset( $_GET['tab'] ) || 'dashboard' == $_GET['tab'] ) {
				$input .= '<a class="ui button basic icon" onclick="mainwp_ssl_monitor_sites_screen_options(); return false;" data-inverted="" data-position="bottom right" href="#" target="_blank" data-tooltip="' . esc_html__( 'Page Settings', 'mainwp' ) . '"><i class="cog icon"></i></a>';
			}
		}
		return $input;
	}

	/**
	 * Method handle_sites_screen_settings()
	 *
	 * Handle sites screen settings
	 */
	public function handle_sites_screen_settings() {
		if ( isset( $_POST['wp_nonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wp_nonce'] ), 'SSLMonitorSitesScrOptions' ) ) {
			$show_cols = array();
			foreach ( array_map( 'sanitize_text_field', wp_unslash( $_POST ) ) as $key => $val ) {
				if ( false !== strpos( $key, 'mainwp_show_column_' ) ) {
					$col               = str_replace( 'mainwp_show_column_', '', $key );
					$show_cols[ $col ] = 1;
				}
			}
			if ( isset( $_POST['show_columns_name'] ) ) {
				foreach ( array_map( 'sanitize_text_field', wp_unslash( $_POST['show_columns_name'] ) ) as $col ) {
					if ( ! isset( $show_cols[ $col ] ) ) {
						$show_cols[ $col ] = 0; // uncheck, hide columns.
					}
				}
			}
			$user = wp_get_current_user();
			if ( $user ) {
				update_user_option( $user->ID, 'mainwp_settings_show_ssl_monitor_sites_columns', $show_cols, true );
			}
		}
	}

	/**
	 * REST API SSL Info
	 *
	 * Handles the REST API sites SSL info.
	 *
	 * @param array $websites Child sites.
	 *
	 * @return array $data Reports data.
	 */
	public static function handle_rest_api_sites_ssl_info( $websites ) {
		$data = array();
		if ( ! empty( $websites ) ) {
			foreach ( $websites as $website ) {
				$website_id = $website['id'];

				$result = array();

				$ssl_data = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $website_id );

				$result['id']   = $website['id'];
				$result['name'] = $website['name'];
				$result['url']  = $website['url'];

				if ( ! empty( $ssl_data ) ) {
					$result['ssl_name']           = isset( $ssl_data->name ) ? $ssl_data->name : '';
					$result['issuer_o']           = isset( $ssl_data->issuer_o ) ? $ssl_data->issuer_o : '';
					$result['issuer_cn']          = isset( $ssl_data->issuer_cn ) ? $ssl_data->issuer_cn : '';
					$result['issuer_c']           = isset( $ssl_data->issuer_c ) ? $ssl_data->issuer_c : '';
					$result['subject_c']          = isset( $ssl_data->subject_c ) ? $ssl_data->subject_c : '';
					$result['subject_st']         = isset( $ssl_data->subject_st ) ? $ssl_data->subject_st : '';
					$result['subject_l']          = isset( $ssl_data->subject_l ) ? $ssl_data->subject_l : '';
					$result['subject_o']          = isset( $ssl_data->subject_o ) ? $ssl_data->subject_o : '';
					$result['subject_cn']         = isset( $ssl_data->subject_cn ) ? $ssl_data->subject_cn : '';
					$result['valid_from']         = isset( $ssl_data->valid_from ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_from ) ) : '';
					$result['valid_to']           = isset( $ssl_data->valid_to ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ) : '';
					$result['subject_alt_name']   = isset( $ssl_data->subject_alt_name ) ? $ssl_data->subject_alt_name : '';
					$result['hash']               = isset( $ssl_data->hash ) ? $ssl_data->hash : '';
					$result['version']            = isset( $ssl_data->version ) ? $ssl_data->version : '';
					$result['serial_number']      = isset( $ssl_data->serial_number ) ? $ssl_data->serial_number : '';
					$result['serial_number_hex']  = isset( $ssl_data->serial_number_hex ) ? $ssl_data->serial_number_hex : '';
					$result['signature_type_sn']  = isset( $ssl_data->signature_type_sn ) ? $ssl_data->signature_type_sn : '';
					$result['signature_type_ln']  = isset( $ssl_data->signature_type_ln ) ? $ssl_data->signature_type_ln : '';
					$result['signature_type_nid'] = isset( $ssl_data->signature_type_nid ) ? $ssl_data->signature_type_nid : '';

					$data['resutls'][ $website['id'] ] = $result;
				}
			}
		} else {
			$data['error'] = __( 'SSL info not found.', 'mainwp-ssl-monitor-extension' );
		}
		return $data;
	}

	/**
	 * WP CLI SSL Info
	 *
	 * Handles the WP CLI SSL info.
	 *
	 * @param array $website Child site.
	 */
	public static function handle_wp_cli_sites_ssl_info( $website ) {

		$website_id = $website['id'];

		$ssl_data = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $website_id );

		\WP_CLI::line( ' -> ' . $website['name'] . ' (' . $website['url'] . ')' );

		if ( ! empty( $ssl_data ) ) {

			$ssl_name           = isset( $ssl_data->name ) ? $ssl_data->name : '';
			$issuer_o           = isset( $ssl_data->issuer_o ) ? $ssl_data->issuer_o : '';
			$issuer_cn          = isset( $ssl_data->issuer_cn ) ? $ssl_data->issuer_cn : '';
			$issuer_c           = isset( $ssl_data->issuer_c ) ? $ssl_data->issuer_c : '';
			$subject_c          = isset( $ssl_data->subject_c ) ? $ssl_data->subject_c : '';
			$subject_st         = isset( $ssl_data->subject_st ) ? $ssl_data->subject_st : '';
			$subject_l          = isset( $ssl_data->subject_l ) ? $ssl_data->subject_l : '';
			$subject_o          = isset( $ssl_data->subject_o ) ? $ssl_data->subject_o : '';
			$subject_cn         = isset( $ssl_data->subject_cn ) ? $ssl_data->subject_cn : '';
			$valid_from         = isset( $ssl_data->valid_from ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_from ) ) : '';
			$valid_to           = isset( $ssl_data->valid_to ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ) : '';
			$subject_alt_name   = isset( $ssl_data->subject_alt_name ) ? $ssl_data->subject_alt_name : '';
			$hash               = isset( $ssl_data->hash ) ? $ssl_data->hash : '';
			$version            = isset( $ssl_data->version ) ? $ssl_data->version : '';
			$serial_number      = isset( $ssl_data->serial_number ) ? $ssl_data->serial_number : '';
			$serial_number_hex  = isset( $ssl_data->serial_number_hex ) ? $ssl_data->serial_number_hex : '';
			$signature_type_sn  = isset( $ssl_data->signature_type_sn ) ? $ssl_data->signature_type_sn : '';
			$signature_type_ln  = isset( $ssl_data->signature_type_ln ) ? $ssl_data->signature_type_ln : '';
			$signature_type_nid = isset( $ssl_data->signature_type_nid ) ? $ssl_data->signature_type_nid : '';

			\WP_CLI::line( __( 'Name: ', 'mainwp-ssl-monitor-extension' ) . $ssl_name );
			\WP_CLI::line( __( 'Issuer Organization: ', 'mainwp-ssl-monitor-extension' ) . $issuer_o );
			\WP_CLI::line( __( 'Issuer Common Name: ', 'mainwp-ssl-monitor-extension' ) . $issuer_cn );
			\WP_CLI::line( __( 'Issuer Country: ', 'mainwp-ssl-monitor-extension' ) . $issuer_c );
			\WP_CLI::line( __( 'Subject (C): ', 'mainwp-ssl-monitor-extension' ) . $subject_c );
			\WP_CLI::line( __( 'Subject (ST): ', 'mainwp-ssl-monitor-extension' ) . $subject_st );
			\WP_CLI::line( __( 'Subject (L): ', 'mainwp-ssl-monitor-extension' ) . $subject_l );
			\WP_CLI::line( __( 'Subject (O): ', 'mainwp-ssl-monitor-extension' ) . $subject_o );
			\WP_CLI::line( __( 'Subject (CN): ', 'mainwp-ssl-monitor-extension' ) . $subject_cn );
			\WP_CLI::line( __( 'Valid From: ', 'mainwp-ssl-monitor-extension' ) . $valid_from );
			\WP_CLI::line( __( 'Valid To: ', 'mainwp-ssl-monitor-extension' ) . $valid_to );
			\WP_CLI::line( __( 'Subject Alternative Name: ', 'mainwp-ssl-monitor-extension' ) . $subject_alt_name );
			\WP_CLI::line( __( 'Hash: ', 'mainwp-ssl-monitor-extension' ) . $hash );
			\WP_CLI::line( __( 'Version: ', 'mainwp-ssl-monitor-extension' ) . $version );
			\WP_CLI::line( __( 'Serial Number: ', 'mainwp-ssl-monitor-extension' ) . $serial_number );
			\WP_CLI::line( __( 'Serial Number Hex: ', 'mainwp-ssl-monitor-extension' ) . $serial_number_hex );
			\WP_CLI::line( __( 'Signature Type SN: ', 'mainwp-ssl-monitor-extension' ) . $signature_type_sn );
			\WP_CLI::line( __( 'Signature Type LN: ', 'mainwp-ssl-monitor-extension' ) . $signature_type_ln );
			\WP_CLI::line( __( 'Signature Type NID: ', 'mainwp-ssl-monitor-extension' ) . $signature_type_nid );
			\WP_CLI::line( '' );

		} else {
			\WP_CLI::line( __( 'SSL data not found.', 'mainwp-ssl-monitor-extension' ) );
		}
	}

	/**
	 * Hooks the section help content to the Help Sidebar element.
	 */
	public static function mainwp_help_content() {
		if ( isset( $_GET['page'] ) && ( 'Extensions-Mainwp-Ssl-Monitor-Extension' === $_GET['page'] ) ) {
			?>
			<p><?php esc_html_e( 'If you need help with the SSL Monitor extension, please review following help documents', 'mainwp' ); ?></p>
			<div class="ui relaxed bulleted list">
				<div class="item"><a href="https://kb.mainwp.com/" target="_blank">How to use the Extension</a></div>
				<?php
				/**
				 * Action: mainwp_ssl_monitor_help_item
				 *
				 * Fires at the bottom of the help articles list in the Help sidebar on the Themes page.
				 *
				 * Suggested HTML markup:
				 *
				 * <div class="item"><a href="Your custom URL">Your custom text</a></div>
				 *
				 * @since 4.1
				 */
				do_action( 'mainwp_ssl_monitor_help_item' );
				?>
			</div>
			<?php
		}
	}

	/**
	 * Hook hook_mainwp_specific_action_logs.
	 *
	 * Hook sites table sortable columns.
	 *
	 * @return mixed $sortable_columns.
	 */
	public function hook_mainwp_specific_action_logs( $inputs ) {
		$inputs[ MAINWP_SSL_MONITOR_LOG_PRIORITY ] = __( 'SSL Monitor', 'mainwp-ssl-monitor-extension' );
		return $inputs;
	}
}
