<?php

namespace MainWP\Extensions\SSL_Monitor;

class MainWP_SSL_Monitor_Hooks {

	/**
	 * Public static variable to hold the single instance of the class.
	 *
	 * @static
	 *
	 * @var mixed Default null
	 */
	public static $instance = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_SSL_Monitor_Hooks
	 */
	static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * Runs each time the class is called.
	 */
	public function __construct() {

	}

	/**
	 * Initiate Hooks
	 *
	 * Initiates hooks for the SSL Monitor extension.
	 */
	public function init() {
		add_action( 'mainwp_delete_site', array( &$this, 'delete_site' ), 10, 1 );
		add_filter( 'mainwp_getsubpages_sites', array( &$this, 'managesites_subpage' ), 10, 1 );
		add_filter( 'mainwp_sitestable_getcolumns', array( $this, 'manage_sites_column' ), 10 );
		add_filter( 'mainwp_sitestable_item', array( $this, 'manage_sites_item' ), 10 );
		add_filter( 'mainwp_monitoring_sitestable_getcolumns', array( $this, 'manage_sites_column' ), 10 );
		add_filter( 'mainwp_monitoring_sitestable_item', array( $this, 'manage_sites_item' ), 10 );
		add_filter( 'mainwp_notification_types', array( &$this, 'hook_notification_types' ), 10, 2 );
		add_filter( 'mainwp_default_emails_fields', array( &$this, 'hook_default_emails_fields' ), 10, 5 );
		add_filter( 'mainwp_notification_type_desc', array( &$this, 'hook_notification_type_desc' ), 10, 2 );

		add_filter( 'mainwp_get_notification_template_name_by_type', array( &$this, 'hook_get_template_name_by_type' ), 10, 2 );
		add_filter( 'mainwp_default_template_locate', array( &$this, 'hook_default_notification_template_locate' ), 10, 5 );
		add_filter( 'minwp_notification_template_copy_message', array( &$this, 'hook_notification_template_copy_message' ), 10, 4 );
		add_filter( 'mainwp_default_template_source_dir', array( &$this, 'hook_default_template_source_dir' ), 10, 2 );
		add_filter( 'mainwp_ssl_monitor_get_data', array( &$this, 'get_ssl_monitor_reports_data' ), 10, 5 );
	}

	/**
	 * Delete SSL Monitor Data
	 *
	 * Deletes the SSL Monitor data for a child site.
	 *
	 * @param object $website Child site.
	 */
	public function delete_site( $website ) {
		if ( $website ) {
			MainWP_SSL_Monitor_DB::get_instance()->delete_ssl_monitor( 'site_id', $website->id );
		}
	}

	/**
	 * Site SSL Monitor page
	 *
	 * Creates the SSL Monitor page for each child site.
	 *
	 * @param array $subPage Subpage data.
	 *
	 * @return array $subPage Subpage data.
	 */
	public function managesites_subpage( $subPage ) {
		$subPage[] = array(
			'title'       => __( 'SSL Monitor', 'mainwp-ssl-monitor-extension' ),
			'slug'        => 'SSLMonitor',
			'sitetab'     => true,
			'menu_hidden' => true,
			'callback'    => array( MainWP_SSL_Monitor_Admin::class, 'render_extension_page' ),
		);
		return $subPage;
	}

	/**
	 * Manage Sites Column
	 *
	 * Adds the custom column in the Manage Sites and Monitoring tables.
	 *
	 * @param array $columns Table comlumns.
	 *
	 * @return array $columns Table comlumns.
	 */
	public function manage_sites_column( $columns ) {
		$columns['ssl-monitor'] = __( 'SSL Status', 'mainwp-ssl-monitor-extension' );
		return $columns;
	}

	/**
	 * Manage Sites Item
	 *
	 * Adds the custom column data in the Manage Sites and Monitoring tables.
	 *
	 * @param array $item Site comlumn data.
	 *
	 * @return array $item Site comlumn data.
	 */
	public function manage_sites_item( $item ) {
		$ssl_data = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $item['id'] );
		$status   = '';

		if ( ! empty( $ssl_data ) && is_object( $ssl_data ) ) {
			$valid_from = isset( $ssl_data->valid_from ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_from ) ) : '';
			$valid_to   = isset( $ssl_data->valid_to ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ) : '';
			$last_check = isset( $ssl_data->last_check ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->last_check ) ) : '';

			$expires = round( ( intval( $ssl_data->valid_to ) - time() ) / ( 60 * 60 * 24 ) );

			$status = '<span class="ui green fluid mini center aligned label" data-tooltip="Exires in ' . $expires . ' days" data-position="left center" data-inverted="">' . __( 'Valid', 'mainwp-ssl-monitor-extension' ) . '</span>';

			if ( 0 > $expires ) {
				$status = '<span class="ui red fluid mini center aligned label">' . __( 'Expired', 'mainwp-ssl-monitor-extension' ) . '</span>';
			}
		}

		if ( MainWP_SSL_Monitor_Utility::starts_with( $item['url'], 'https://' ) ) {
			$item['ssl-monitor'] = $status;
		} else {
			$item['ssl-monitor'] = 'N/A';
		}

		return $item;
	}

	/**
	 * Hook notification types
	 *
	 * Hook notification types.
	 *
	 * @param array  $types notification types.
	 * @param string $type  notification type input.
	 *
	 * @return array $types notification types.
	 */
	public function hook_notification_types( $types, $type = '' ) {
		if ( ! is_array( $types ) ) {
			$types = array();
		}
		$types['ssl_monitor_notification_email'] = __( 'SSL Monitor Notification Email', 'mainwp-ssl-monitor-extension' );
		return $types;
	}

	/**
	 * Hook default emails fields
	 *
	 * Hook default emails fields.
	 *
	 * @param array  $fields emails fields.
	 * @param array  $recipients recipients.
	 * @param string $type notification type input.
	 * @param string $field emails fields.
	 * @param bool   $general general or individual.
	 *
	 * @return array $fields emails fields.
	 */
	public function hook_default_emails_fields( $fields, $recipients, $type, $field, $general ) {
		if ( ! is_array( $fields ) ) {
			$fields = array();
		}
		$disable                                  = $general ? 0 : 1;
		$fields['ssl_monitor_notification_email'] = array(
			'disable'    => $disable,
			'recipients' => $recipients,
			'subject'    => $general ? 'MainWP SSL Monitor Notification' : 'MainWP SSL Monitor Notification for [site.name]',
			'heading'    => $general ? 'MainWP SSL Monitor Notification' : 'MainWP SSL Monitor Notification for [site.name]',
		);
		return $fields;
	}

	/**
	 * Hook notification type description
	 *
	 * Hook notification type description.
	 *
	 * @param string $desc notification settings description.
	 * @param string $type notification type input.
	 *
	 * @return array $desc notification description.
	 */
	public function hook_notification_type_desc( $desc, $type ) {
		if ( 'ssl_monitor_notification_email' == $type ) {
			$desc = __( 'SSL Monitor Notification Email', 'mainwp-ssl-monitor-extension' );
		}
		return $desc;
	}

	/**
	 * Hook get template name by type
	 *
	 * Hook get template name by type.
	 *
	 * @param string $template_name template name.
	 * @param string $type notification type input.
	 *
	 * @return array $template_name template name.
	 */
	public function hook_get_template_name_by_type( $template_name, $type ) {
		if ( 'ssl_monitor_notification_email' == $type ) {
			$template_name = 'emails/mainwp-ssl-monitor-notification-email.php';
		}
		return $template_name;
	}

	/**
	 * Hook default notification template locate
	 *
	 * Hook default notification template locate.
	 *
	 * @param string $default_file default template file.
	 * @param string $template template.
	 * @param string $default_dir default directory.
	 * @param string $type notification type input.
	 * @param int    $siteid site id.
	 *
	 * @return array $template_name template name.
	 */
	public function hook_default_notification_template_locate( $default_file, $template, $default_dir, $type, $siteid ) {
		if ( 'ssl_monitor_notification_email' == $type ) {
			$default_file = MAINWP_SSL_MONITOR_PLUGIN_DIR . '/templates/emails/mainwp-ssl-monitor-notification-email.php';
		}
		return $default_file;
	}

	/**
	 * Hook default notification template locate
	 *
	 * Hook default notification template locate.
	 *
	 * @param string $message copy message.
	 * @param string $templ template.
	 * @param string $type notification type input.
	 * @param bool   $overwrited overwrited template or not.
	 *
	 * @return array $template_name template name.
	 */
	public function hook_notification_template_copy_message( $message, $templ, $type, $overwrited ) {
		if ( 'ssl_monitor_notification_email' == $type ) {
			$message = $overwrited ? esc_html__( 'This template has been overridden and can be found in:', 'mainwp-ssl-monitor-extension' ) . ' <code>wp-content/uploads/mainwp/templates/' . $templ . '</code>' : esc_html__( 'To overwrite and edit this email template copy:', 'mainwp-ssl-monitor-extension' ) . ' <code>mainwp-ssl-monitor-extension/templates/' . $templ . '</code> ' . esc_html__( 'to the folder:', 'mainwp-ssl-monitor-extension' ) . ' <code>wp-content/uploads/mainwp/templates/' . $templ . '</code>';
		}
		return $message;
	}

	/**
	 * Hook default notification template source directory.
	 *
	 * Hook default notification template source directory.
	 *
	 * @param string $template_path source directory.
	 * @param string $template_base_name notification type input.
	 *
	 * @return string $template_path source directory.
	 */
	public function hook_default_template_source_dir( $template_path, $template_base_name = '' ) {
		if ( 'emails/mainwp-ssl-monitor-notification-email.php' == $template_base_name ) {
			$template_path = MAINWP_SSL_MONITOR_PLUGIN_DIR . '/templates/';
		}
		return $template_path;
	}

	/**
	 * Get SSL Certificate Data
	 *
	 * @param string $input     Input value.
	 * @param int    $site_id   Child Site ID.
	 *
	 * @return array Report data.
	 */
	public function get_ssl_monitor_reports_data( $input, $site_id ) {

		if ( empty( $site_id ) ) {
			return $input;
		}

		$ssl_data = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $site_id );

		if ( ! empty( $ssl_data ) && is_object( $ssl_data ) ) {
			$common_name = isset( $ssl_data->subject_cn ) ? $ssl_data->subject_cn : '';
			$issuer      = isset( $ssl_data->issuer_o ) ? $ssl_data->issuer_o : '';
			$valid_from  = isset( $ssl_data->valid_from ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_from ) ) : '';
			$valid_to    = isset( $ssl_data->valid_to ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->valid_to ) ) : '';
			$last_check  = isset( $ssl_data->last_check ) ? MainWP_SSL_Monitor_Utility::format_datestamp( MainWP_SSL_Monitor_Utility::get_timestamp( $ssl_data->last_check ) ) : '';

			$expires = round( ( intval( $ssl_data->valid_to ) - time() ) / ( 60 * 60 * 24 ) );

			$status = __( 'Valid', 'mainwp-ssl-monitor-extension' );

			if ( 0 > $expires ) {
				$status = __( 'Expired', 'mainwp-ssl-monitor-extension' );
			}

			$input['ssl.monitor.name']       = $common_name;
			$input['ssl.monitor.issuer']     = $issuer;
			$input['ssl.monitor.valid.from'] = $valid_from;
			$input['ssl.monitor.valid.to']   = $valid_to;
			$input['ssl.monitor.expires']    = $expires;
			$input['ssl.monitor.status']     = $status;
			$input['ssl.monitor.last.check'] = $last_check;

		}

		$input = apply_filters( 'mainwp_ssl_monitor_reports_data', $input, $site_id );

		return $input;
	}

}
