<?php
namespace MainWP\Extensions\SSL_Monitor;

class MainWP_SSL_Monitor_Schedule {

	/**
	 * Static variable to hold the single instance of the class.
	 *
	 * @static
	 *
	 * @var mixed Default null
	 */
	static $instance = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_SSL_Monitor_Schedule
	 */
	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Constructor
	 *
	 * Runs each time the class is called.
	 */
	public function __construct() {
	}

	/**
	 * Initiate Hooks
	 *
	 * Initiates hooks for the Domain Monitor extension.
	 */
	public function init() {
		add_filter( 'cron_schedules', array( $this, 'cron_schedules' ) );
		add_action( 'mainwp_ssl_monitor_action_cron_start', array( $this, 'handle_ssl_monitor_cron_start' ), 10, 2 );
		add_action( 'mainwp_ssl_monitor_action_cron_continue', array( $this, 'handle_ssl_monitor_cron_continue' ), 10, 2 );
		add_action( 'mainwp_cron_jobs_list', array( $this, 'cron_job_info' ) );
		$this->init_jobs();
	}

	/**
	 * Init cron jobs.
	 *
	 * Init cron jobs alwys to support disable general schedule but enable individual schedules.
	 */
	public function init_jobs( $old_interval = false ) {

		$dashboard_use_schedule = get_option( 'mainwp_wp_cron' );
		$dashboard_use_schedule = apply_filters( 'mainwp_ssl_monitor_enable_schedule', $dashboard_use_schedule ); // to support enable/disable jobs.

		$recheck_interval = 0;
		if ( $old_interval ) {
			$recheck_interval = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'frequency' );
		}

		$local_timestamp = MainWP_SSL_Monitor_Utility::get_timestamp();
		$today_end   = strtotime( date("Y-m-d 23:59:59", $local_timestamp ) ) ; // phpcs:ignore -- to localtime.

		$sch_time = $today_end;

		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			$sch_time = time() + 2 * MINUTE_IN_SECONDS;
		}

		$init_cron_jobs = false;
		$next_sche      = 0;

		$sched = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_start' );
		// init start check daily.
		if ( $dashboard_use_schedule && ! $sched ) {
			wp_schedule_event( $sch_time, 'mainwp_ssl_monitor_start_interval', 'mainwp_ssl_monitor_action_cron_start' );
			$next_sche      = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_start' );
			$init_cron_jobs = true;
		} elseif ( $dashboard_use_schedule && $old_interval && $recheck_interval !== $old_interval ) {
			wp_unschedule_event( $sched, 'mainwp_ssl_monitor_action_cron_start' );
			wp_schedule_event( $sch_time, 'mainwp_ssl_monitor_start_interval', 'mainwp_ssl_monitor_action_cron_start' );
			$next_sche      = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_start' );
			$init_cron_jobs = true;
		}

		$sched = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_continue' );
		// init continue check 5 minutely.
		if ( $dashboard_use_schedule && ! $sched ) {
			wp_schedule_event( $sch_time + 5 * MINUTE_IN_SECONDS, 'mainwp_ssl_monitor_continue_interval', 'mainwp_ssl_monitor_action_cron_continue' );
		} elseif ( $dashboard_use_schedule && $old_interval && $recheck_interval !== $old_interval ) {
			wp_unschedule_event( $sched, 'mainwp_ssl_monitor_action_cron_continue' );
			wp_schedule_event( $sch_time + 5 * MINUTE_IN_SECONDS, 'mainwp_ssl_monitor_continue_interval', 'mainwp_ssl_monitor_action_cron_continue' );
		}

		// init jobs logging.
		if ( $init_cron_jobs ) {
			MainWP_SSL_Monitor_Utility::log_info( 'INIT cron jobs' );
			if ( $next_sche ) {
				MainWP_SSL_Monitor_Utility::log_info( 'next schedule :: ' . gmdate( 'Y-m-d H:i:s', $next_sche ) );
			}
		}

		if ( ! $dashboard_use_schedule ) {
			// remove jobs if disabled.
			$sched = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_start' );
			if ( $sched ) {
				wp_unschedule_event( $sched, 'mainwp_ssl_monitor_action_cron_start' );
				MainWP_SSL_Monitor_Utility::log_info( 'STOP cron jobs' );
			}
			$sched = wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_continue' );
			if ( $sched ) {
				wp_unschedule_event( $sched, 'mainwp_ssl_monitor_action_cron_continue' );
			}
		}
	}

	/**
	 * Cron Schedules
	 *
	 * Defines cron schedules.
	 *
	 * @param array $schedules Cron schedules.
	 *
	 * @return array $schedules Cron schedules.
	 */
	public function cron_schedules( $schedules ) {
		if ( ! isset( $schedules['mainwp_ssl_monitor_start_interval'] ) ) {
			$schedules['mainwp_ssl_monitor_start_interval'] = array(
				'interval' => DAY_IN_SECONDS, // start audit daily to support disable general schedule but enable individual schedule.
				'display'  => __( 'MainWP SSL Monitor Continue Interval', 'mainwp-ssl-monitor-extension' ),
			);
		}

		if ( ! isset( $schedules['mainwp_ssl_monitor_continue_interval'] ) ) {
			$schedules['mainwp_ssl_monitor_continue_interval'] = array(
				'interval' => 5 * MINUTE_IN_SECONDS, // continue every 5 minutes.
				'display'  => __( 'Once Every 5 Minutes', 'mainwp-ssl-monitor-extension' ),
			);
		}

		return $schedules;
	}


	/**
	 * Start Cron
	 *
	 * Starts the cron jobs.
	 */
	public function handle_ssl_monitor_cron_start() {
		$this->cron_run_start_check();
	}

	/**
	 * Start Cron
	 *
	 * Starts the cron jobs.
	 */
	public function cron_run_start_check() {
		MainWP_SSL_Monitor_Utility::log_info( 'START init urls checking' );
		MainWP_SSL_Monitor_DB_Schedule::get_instance()->update_to_start_auto_check(); // to check frequency of all ssl to check.
		update_option( 'mainwp_ssl_monitor_running_cron_check', true ); // running cron check.
	}

	/**
	 * Method handle_ssl_monitor_cron_continue().
	 *
	 * Cron continue SSL monitor.
	 *
	 * Handle
	 */
	public function handle_ssl_monitor_cron_continue() {
		return $this->cron_run_continue_check();
	}

	/**
	 * Method cron_run_continue_check().
	 *
	 * Cron continue SSL monitor audits every 5 minutes,
	 * and/or start and continue send notification alerts.
	 *
	 * @return void.
	 */
	public function cron_run_continue_check() {
		$ssl_sites = MainWP_SSL_Monitor_DB_Schedule::get_instance()->get_to_continue_auto_check();

		if ( ! empty( $ssl_sites ) ) {

			$general_schedule = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'use_schedule' );
			$general_schedule = intval( $general_schedule );
			$general_interval = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'frequency' );

			$bulk_run  = 4;
			$count_run = 0;

			MainWP_SSL_Monitor_Utility::log_info( 'CONTINUE total urls to check :: ' . count( $ssl_sites ) );
			foreach ( $ssl_sites as $ssl_site ) {
				$site_id            = $ssl_site->site_id;
				$id                 = $ssl_site->id;
				$site_url           = $ssl_site->URL;
				$overwrite          = intval( $ssl_site->overwrite );
				$indiv_use_schedule = intval( $ssl_site->use_schedule );
				$interval           = intval( $general_interval );
				$skip_this          = false;

				if ( $overwrite ) {
					if ( empty( $indiv_use_schedule ) ) {
						$skip_this = true;
					} else {
						$settings = ( '' != $ssl_site->settings ) ? json_decode( $ssl_site->settings, true ) : array();
						$settings = is_array( $settings ) ? $settings : array();
						if ( isset( $settings['frequency'] ) ) {
							$interval = intval( $settings['frequency'] );
						}
					}
				} elseif ( empty( $general_schedule ) ) {
						$skip_this = true;
				}

				$last_check = is_numeric( $ssl_site->last_check ) ? intval( $ssl_site->last_check ) : 0;
				if ( time() < $last_check + $interval ) {
					$skip_this = true;
				}

				if ( $skip_this ) {
					continue;
				}

				// update 'completed_auto_check' so it will not included in query for next continue step.
				MainWP_SSL_Monitor_DB_Schedule::get_instance()->update_completed_auto_check_for_site( $site_id );

				MainWP_SSL_Monitor_Utility::get_ssl_certificate_info( $site_url, $id, $site_id );

				++$count_run;
				if ( $count_run >= $bulk_run ) {
					break;
				}
			}
		} else {
			$running = get_option( 'mainwp_ssl_monitor_running_cron_check' );
			if ( ! $running ) {
				$this->handle_continue_alert(); // continue alert.
			} else {
				update_option( 'mainwp_ssl_monitor_running_cron_check', false );
				MainWP_SSL_Monitor_Utility::log_info( 'FINISHED urls check' );
				self::handle_start_alert(); // start running alert after running audit/check finished.
			}
		}
	}

	/**
	 * Continue Alert
	 *
	 * @param bool $debug Debugging or not.
	 *
	 * Sends emaail alerts when SSL Cert is about to expire.
	 */
	public function handle_continue_alert( $debug = false ) {
		$email_settings = apply_filters( 'mainwp_notification_get_settings', array(), 'ssl_monitor_notification_email' );
		if ( empty( $email_settings ) || ! empty( $email_settings['disable'] ) ) {
			return;
		}
		self::cron_notification_continue( $email_settings, $debug );
	}

	/**
	 * SSL Monitor Alert
	 *
	 * Sends emaail alerts when SSL Cert is about to expire.
	 */
	public static function handle_start_alert() {
		$email_settings = apply_filters( 'mainwp_notification_get_settings', array(), 'ssl_monitor_notification_email' );
		if ( empty( $email_settings ) || ! empty( $email_settings['disable'] ) ) {
			return;
		}
		MainWP_SSL_Monitor_Utility::log_info( 'START alert' );
		MainWP_SSL_Monitor_DB_Schedule::get_instance()->update_to_start_auto_alert(); // start daily to check interval for continue.
	}


	/**
	 * Continue notification.
	 *
	 * @param array $email_settings email settings.
	 * @param bool  $debug Debugging or not.
	 *
	 * Sends emaail alerts when SSL Cert is about to expire.
	 */
	public static function cron_notification_continue( $email_settings, $debug = false ) {

		$time              = time();
		$general_threshold = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'notification_threshold' );
		$interval          = MainWP_SSL_Monitor_Utility::get_instance()->get_option( 'frequency' );
		$interval          = intval( $interval );
		// use two type of alerts:  general/individual frequency
		$alert_general_running = get_option( 'mainwp_ssl_monitor_cron_general_alert_running', false );
		$lastalert_general     = (int) get_option( 'mainwp_ssl_monitor_cron_general_lastalert' );

		$alert_individual_running = get_option( 'mainwp_ssl_monitor_cron_individual_alert_running', false );
		$lastalert_individual     = (int) get_option( 'mainwp_ssl_monitor_cron_individual_lastalert' );

		if ( empty( $alert_general_running ) ) {
			if ( empty( $lastalert_general ) || ( $time > $lastalert_general + $interval ) ) {
				update_option( 'mainwp_ssl_monitor_cron_general_lastalert', $time );
				update_option( 'mainwp_ssl_monitor_cron_general_alert_running', 'yes' );
				$alert_general_running = 'yes';
			}
		}

		$interval_individual = 86400; // individual

		if ( empty( $alert_individual_running ) ) {
			if ( empty( $lastalert_individual ) || ( $time > $lastalert_individual + $interval_individual ) ) {
				update_option( 'mainwp_ssl_monitor_cron_individual_lastalert', $time );
				update_option( 'mainwp_ssl_monitor_cron_individual_alert_running', 'yes' );
				$alert_individual_running = 'yes';
			}
		}

		if ( empty( $alert_general_running ) && empty( $alert_individual_running ) ) {
			return;
		}

			// HEREREERERER
		$ssl_alerts = MainWP_SSL_Monitor_DB_Schedule::get_instance()->get_to_continue_auto_alert();

		if ( empty( $ssl_alerts ) ) {
			// all completed alert.
			$log_finished = false;
			if ( ! empty( $alert_general_running ) ) {
				delete_option( 'mainwp_ssl_monitor_cron_general_alert_running' ); // stop running alert.
				MainWP_SSL_Monitor_Utility::log_info( 'FINISHED general alert' );
			}
			if ( ! empty( $alert_individual_running ) ) {
				delete_option( 'mainwp_ssl_monitor_cron_individual_alert_running' ); // stop running alert.
				MainWP_SSL_Monitor_Utility::log_info( 'FINISHED individual alert' );
			}
			return;
		}

		MainWP_SSL_Monitor_Utility::log_info( 'CONTINUE total urls to alerts :: ' . count( $ssl_alerts ) );

		$count_run = 0;
		$bulk_run  = 4;

		if ( is_array( $ssl_alerts ) ) {
			foreach ( $ssl_alerts as $ssl_alert ) {

				if ( empty( $ssl_alert->site_id ) ) {
					continue;
				}

				if ( $ssl_alert->overwrite && empty( $alert_individual_running ) ) { // individual settings.
					continue;
				}

				if ( empty( $ssl_alert->overwrite ) && empty( $alert_general_running ) ) { // general settings.
					continue;
				}

				// update 'completed_auto_alert' so it will not included in query for next continue step.
				MainWP_SSL_Monitor_DB_Schedule::get_instance()->update_completed_auto_alert_for_site( $ssl_alert->site_id );

				if ( $ssl_alert->overwrite ) {
					$scheduled = $ssl_alert->use_schedule;
					if ( empty( $scheduled ) ) {
						continue;
					}
					$settings                    = json_decode( $ssl_alert->settings, true );
					$settings                    = is_array( $settings ) ? $settings : array();
					$site_notification_threshold = isset( $settings['notification_threshold'] ) ? $settings['notification_threshold'] : 0;
				} else {
					$site_notification_threshold = $general_threshold;
				}

				if ( ! empty( $site_notification_threshold ) ) {
					$last_alert = $ssl_alert->last_alert;
					$expires    = round( ( $ssl_alert->valid_to - time() ) / ( 60 * 60 * 24 ) );
					if ( $expires < $site_notification_threshold ) {
						if ( $last_alert + 24 * 3600 < $time ) {
							$sent   = self::send_alert_mail( $ssl_alert, $email_settings );
							$update = array(
								'last_alert' => $time,
								'site_id'    => $ssl_alert->site_id,
							);
							MainWP_SSL_Monitor_DB_Schedule::get_instance()->update_ssl_monitor( $update );
							if ( $sent ) {
								MainWP_SSL_Monitor_Utility::log_info( ' :: alert :: success :: ' );
							} else {
								MainWP_SSL_Monitor_Utility::log_info( ' :: alert :: failed :: ' );
							}
							++$count_run;
						}
					}

					if ( $count_run >= $bulk_run ) {
						return;
					}
				}
			}
		}
	}

	/**
	 * Send Email Alert
	 *
	 * Sends email notifications when SSL Certificate is about to expire.
	 *
	 * @param array  $ssl_alert      SSL data for a site.
	 * @param string $email_settings Email settings.
	 *
	 * @return bool True on succes, false on failure.
	 */
	public static function send_alert_mail( $ssl_alert, $email_settings ) {

		if ( empty( $ssl_alert->site_id ) ) {
			return false;
		}

		$site_id = $ssl_alert->site_id;

		$website = MainWP_SSL_Monitor_Utility::get_websites( $site_id );

		if ( $website && is_array( $website ) ) {
			$website = current( $website );
		}

		if ( ! $website ) {
			return;
		}

		$email = '';

		if ( ! empty( $email_settings['recipients'] ) ) {
			$email .= ',' . $email_settings['recipients']; // send to recipients, individual email settings or general email settings.
		}

		$email = trim( $email, ',' );

		if ( empty( $email ) ) {
			return false;
		}

		$args = array(
			'site_url'   => $website['url'],
			'site_name'  => $website['name'],
			'site_id'    => $site_id,
			'ssl_alert'  => $ssl_alert,
			'last_check' => $ssl_alert->last_check,
			'heading'    => $email_settings['heading'],
		);

		$formated_content = apply_filters( 'mainwp_notification_get_template_content', '', 'emails/mainwp-ssl-monitor-notification-email.php', $args );
		$content_type     = "Content-Type: text/html; charset=\"utf-8\"\r\n";
		$subject          = $email_settings['subject'];
		$sent             = apply_filters( 'mainwp_send_wp_mail', null, $email, $subject, $formated_content, $content_type );

		return $sent;
	}

	/**
	 * Cron Job Info
	 *
	 * Hooks the SSL Monitor cron job info to the Cron Schedules table.
	 */
	public function cron_job_info() {
		$next_run = MainWP_SSL_Monitor_Utility::format_timestamp( MainWP_SSL_Monitor_Utility::get_timestamp( wp_next_scheduled( 'mainwp_ssl_monitor_action_cron_start' ) ) );
		$last_run = MainWP_SSL_Monitor_Utility::format_timestamp( MainWP_SSL_Monitor_Utility::get_timestamp( get_option( 'mainwp_ssl_monitor_action_cron_start' ) ) );
		?>
		<tr>
			<td><?php echo __( 'Start SSL Certificate check', 'mainwp-ssl-monitor-extension' ); ?></td>
			<td><?php echo 'mainwp_ssl_monitor_action_cron_start'; ?></td>
			<td><?php echo __( 'Once every 5 minutes', 'mainwp-ssl-monitor-extension' ); ?></td>
			<td><?php echo $last_run; ?></td>
			<td><?php echo $next_run; ?></td>
		</tr>
		<?php
	}
}
