<?php

/**
 * Class MainWP_SSL_Monitor_Utility
 */
namespace MainWP\Extensions\SSL_Monitor;

class MainWP_SSL_Monitor_Utility {

	public static $instance = null;

	protected $option_handle = 'mainwp_ssl_monitor_options';
	protected $option        = null;

	/**
	 * Get Instance
	 *
	 * Creates public static instance.
	 *
	 * @static
	 *
	 * @return MainWP_SSL_Monitor_Utility
	 */
	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public function __construct() {
		if ( null === $this->option ) {
			$this->option = get_option( $this->option_handle, array() );
		}

		add_action( 'mainwp_ssl_monitor_update_option', array( $this, 'set_option' ), 10, 2 );
	}

	/**
	 * Get Option
	 *
	 * Gets option in Databse.
	 *
	 * @param string $key Option key.
	 * @param mixed  $value Option value.
	 *
	 * @return mixed Retruns option value.
	 */
	public function get_option( $key = null, $default = '' ) {
		if ( isset( $this->option[ $key ] ) ) {
			return $this->option[ $key ];
		}
		return $default;
	}

	/**
	 * Set Option
	 *
	 * Sets option in Databse.
	 *
	 * @param string $key Option key.
	 * @param mixed  $value Option value.
	 *
	 * @return mixed Update option.
	 */
	public function set_option( $key, $value ) {
		$this->option[ $key ] = $value;
		return update_option( $this->option_handle, $this->option );
	}

	/**
	 * Get timestamp.
	 *
	 * @param string $timestamp Holds Timestamp.
	 *
	 * @return float|int Return GMT offset.
	 */
	public static function get_timestamp( $timestamp = false ) {
		if ( false === $timestamp ) {
			$timestamp = time();
		}
		$gmtOffset = get_option( 'gmt_offset' );

		$timestamp = intval( $timestamp );

		return ( $gmtOffset ? ( $gmtOffset * HOUR_IN_SECONDS ) + $timestamp : $timestamp );
	}

	/**
	 * Format timestamp.
	 *
	 * @param string $timestamp Holds Timestamp.
	 * @param bool   $gmt Whether to set as General mountain time. Default: FALSE.
	 *
	 * @return string Return Timestamp.
	 */
	public static function format_timestamp( $timestamp, $gmt = false ) {
		return date_i18n( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $timestamp, $gmt );
	}

	/**
	 * Format datestamp.
	 *
	 * @param string $timestamp Holds Timestamp.
	 * @param bool   $gmt Whether to set as General mountain time. Default: FALSE.
	 *
	 * @return string Return Timestamp.
	 */
	public static function format_datestamp( $timestamp, $gmt = false ) {
		return date_i18n( get_option( 'date_format' ), $timestamp, $gmt );
	}

	/**
	 * Format date.
	 *
	 * @param string $timestamp Holds Timestamp.
	 * @param bool   $gmt Whether to set as General mountain time. Default: FALSE.
	 *
	 * @return string Return Timestamp.
	 */
	public static function format_date( $timestamp, $gmt = false ) {
		return date_i18n( get_option( 'date_format' ), $timestamp, $gmt );
	}


	/**
	 * Method map_fields()
	 *
	 * Map Site.
	 *
	 * @param mixed $website Website to map.
	 * @param mixed $keys Keys to map.
	 * @param bool  $object_output Output format array|object.
	 *
	 * @return object $outputSite Mapped site.
	 */
	public static function map_fields( &$website, $keys, $object_output = false ) {
		$outputSite = array();
		if ( ! empty( $website ) ) {
			if ( is_object( $website ) ) {
				foreach ( $keys as $key ) {
					if ( property_exists( $website, $key ) ) {
						$outputSite[ $key ] = $website->$key;
					}
				}
			} elseif ( is_array( $website ) ) {
				foreach ( $keys as $key ) {
					$outputSite[ $key ] = $website[ $key ];
				}
			}
		}

		if ( $object_output ) {
			return (object) $outputSite;
		} else {
			return $outputSite;
		}
	}

	/**
	 * Convert time to readable format.
	 *
	 * Converts time stamp to more readable format to display time difference.
	 *
	 * @param mixed $time Timestamp.
	 *
	 * @return string Time difference.
	 */
	public function human_timing( $time ) {
		if ( empty( $time ) ) {
			return 'N/A';
		}
		$time = current_time( 'timestamp' ) - $time;

		$tokens = array(
			31536000 => __( 'year', 'mainwp-ssl-monitor-extension' ),
			2592000  => __( 'month', 'mainwp-ssl-monitor-extension' ),
			604800   => __( 'week', 'mainwp-ssl-monitor-extension' ),
			86400    => __( 'day', 'mainwp-ssl-monitor-extension' ),
			3600     => __( 'hour', 'mainwp-ssl-monitor-extension' ),
			60       => __( 'minute', 'mainwp-ssl-monitor-extension' ),
			1        => __( 'second', 'mainwp-ssl-monitor-extension' ),
		);

		foreach ( $tokens as $unit => $text ) {
			if ( $time < $unit ) {
				continue;
			}
			$number_of_units = floor( $time / $unit );

			return $number_of_units . ' ' . $text . ( ( $number_of_units > 1 ) ? __( 's ago', 'mainwp-ssl-monitor-extension' ) : ' ' . __( 'ago', 'mainwp-ssl-monitor-extension' ) );
		}
	}

	/**
	 * Show Info Messages
	 *
	 * Check whenther or not to show the MainWP Message.
	 *
	 * @param string $notice_id Notice ID.
	 *
	 * @return bool False if hidden, true to show.
	 */
	public static function show_mainwp_message( $notice_id ) {
		$status = get_user_option( 'mainwp_notice_saved_status' );
		if ( ! is_array( $status ) ) {
			$status = array();
		}
		if ( isset( $status[ $notice_id ] ) ) {
			return false;
		}
		return true;
	}

	/**
	 * Get Websites
	 *
	 * Gets all child sites through the 'mainwp_getsites' filter.
	 *
	 * @param int $site_id  Child site ID.
	 * @param int $group_id Group ID.
	 *
	 * @return array Child sites array.
	 */
	public static function get_websites( $site_id = null, $group_id = null ) {
		global $mainWPSSLMonitorExtensionActivator;
		return apply_filters( 'mainwp_getsites', $mainWPSSLMonitorExtensionActivator->get_child_file(), $mainWPSSLMonitorExtensionActivator->get_child_key(), $site_id, $group_id );
	}

	/**
	 * Get Websites
	 *
	 * Gets speific child sites or groups through the 'mainwp_getsites' filter.
	 *
	 * @param array $site_ids  Child sites IDs.
	 * @param array $group_ids Groups IDs.
	 *
	 * @return array Child sites array.
	 */
	public static function get_db_sites( $site_ids, $group_ids = array() ) {
		if ( ! is_array( $site_ids ) ) {
			$site_ids = array();
		}

		if ( ! is_array( $group_ids ) ) {
			$group_ids = array();
		}

		if ( ! empty( $site_ids ) || ! empty( $group_ids ) ) {
			global $mainWPSSLMonitorExtensionActivator;
			return apply_filters( 'mainwp_getdbsites', $mainWPSSLMonitorExtensionActivator->get_child_file(), $mainWPSSLMonitorExtensionActivator->get_child_key(), $site_ids, $group_ids );
		}
		return false;
	}

	/**
	 * Get SSL Certificate Info
	 *
	 * Fetches the SSL Certificate info
	 */
	public static function get_ssl_certificate_info( $url, $id, $site_id ) {
		$orignal_parse = parse_url( $url, PHP_URL_HOST );
		if ( ! empty( $orignal_parse ) ) {
			$ssloptions = array(
				'capture_peer_cert'       => true,
				'capture_peer_cert_chain' => true,
				'allow_self_signed'       => true,
				'CN_match'                => $orignal_parse,
				'verify_peer'             => true,
				'SNI_enabled'             => true,
				'SNI_server_name'         => $orignal_parse,
			);
			$get        = stream_context_create( array( 'ssl' => $ssloptions ) );
			$read       = stream_socket_client( 'ssl://' . $orignal_parse . ':443', $errno, $errstr, 30, STREAM_CLIENT_CONNECT, $get );
			$cert       = false;
			$certinfo   = false;
			if ( is_resource( $read ) ) {
				$cert = stream_context_get_params( $read );
			}
			if ( is_array( $cert ) ) {
				$certinfo = openssl_x509_parse( $cert['options']['ssl']['peer_certificate'] );
			}
			self::save_values_formatted( $certinfo, $id, $site_id, $url );
		}
		$message = $url . __( ' SSL check started.', 'mainwp-ssl-monitor-extension' );
		return $message;
	}

	/**
	 * Save Formated Values
	 *
	 * Saves the SSL data to the mainwp_ssl_monitor Database table.
	 *
	 * @param array  $result     SSL Certitificate info report.
	 * @param int    $id         Item ID.
	 * @param int    $site_id    Site ID.
	 * @param string $site_url   Site URL.
	 */
	public static function save_values_formatted( $result, $id, $site_id, $site_url ) {
		if ( ! is_array( $result ) ) {
			return;
		}
		// compatible name.
		global $wpdb;
		$ssl_data = array();

		$ssl_data['URL']                = $site_url;
		$ssl_data['site_id']            = $site_id;
		$ssl_data['name']               = isset( $result['name'] ) ? $result['name'] : '';
		$ssl_data['subject_c']          = isset( $result['subject']['C'] ) ? $result['subject']['C'] : '';
		$ssl_data['subject_st']         = isset( $result['subject']['ST'] ) ? $result['subject']['ST'] : '';
		$ssl_data['subject_l']          = isset( $result['subject']['L'] ) ? $result['subject']['L'] : '';
		$ssl_data['subject_o']          = isset( $result['subject']['O'] ) ? $result['subject']['O'] : '';
		$ssl_data['subject_cn']         = isset( $result['subject']['CN'] ) ? $result['subject']['CN'] : '';
		$ssl_data['hash']               = isset( $result['hash'] ) ? $result['hash'] : '';
		$ssl_data['issuer_c']           = isset( $result['issuer']['C'] ) ? $result['issuer']['C'] : '';
		$ssl_data['issuer_o']           = isset( $result['issuer']['O'] ) ? $result['issuer']['O'] : '';
		$ssl_data['issuer_cn']          = isset( $result['issuer']['CN'] ) ? $result['issuer']['CN'] : '';
		$ssl_data['version']            = isset( $result['version'] ) ? $result['version'] : '';
		$ssl_data['serial_number']      = isset( $result['serialNumber'] ) ? $result['serialNumber'] : '';
		$ssl_data['serial_number_hex']  = isset( $result['serialNumberHex'] ) ? $result['serialNumberHex'] : '';
		$ssl_data['signature_type_sn']  = isset( $result['signatureTypeSN'] ) ? $result['signatureTypeSN'] : '';
		$ssl_data['signature_type_ln']  = isset( $result['signatureTypeLN'] ) ? $result['signatureTypeLN'] : '';
		$ssl_data['signature_type_nid'] = isset( $result['signatureTypeNID'] ) ? $result['signatureTypeNID'] : '';
		$ssl_data['common_name']        = isset( $result['subject']['CN'] ) ? $result['subject']['CN'] : '';
		$ssl_data['valid_from']         = isset( $result['validFrom_time_t'] ) ? $result['validFrom_time_t'] : '';
		$ssl_data['valid_to']           = isset( $result['validTo_time_t'] ) ? $result['validTo_time_t'] : '';
		$ssl_data['subject_alt_name']   = isset( $result['extensions']['subjectAltName'] ) ? $result['extensions']['subjectAltName'] : '';
		$ssl_data['last_check']         = time();

		if ( $id ) {
			$ssl_data['id'] = $id;
		}

		MainWP_SSL_Monitor_DB::get_instance()->update_ssl_monitor( $ssl_data );
	}

	/**
	 * Method get_recheck_intervals().
	 *
	 * Get recheck intervals.
	 */
	public static function get_recheck_intervals() {
		return array(
			// 300     => __( '5 Minute for testing' ),
			86400   => __( 'Once a Day', 'mainwp-ssl-monitor-extension' ),
			604800  => __( 'Once a Week', 'mainwp-ssl-monitor-extension' ),
			1296000 => __( 'Twice a Month', 'mainwp-ssl-monitor-extension' ),
			2592000 => __( 'Once Monthly', 'mainwp-ssl-monitor-extension' ),
		);
	}

	/**
	 * Method starts_with()
	 *
	 * Start of Stack Trace.
	 *
	 * @param mixed $haystack The full stack.
	 * @param mixed $needle The function that is throwing the error.
	 *
	 * @return mixed Needle in the Haystack.
	 */
	public static function starts_with( $haystack, $needle ) {
		return ! strncmp( $haystack, $needle, strlen( $needle ) );
	}

	/**
	 * Method ends_with()
	 *
	 * End of Stack Trace.
	 *
	 * @param mixed $haystack Haystack parameter.
	 * @param mixed $needle Needle parameter.
	 *
	 * @return boolean
	 */
	public static function ends_with( $haystack, $needle ) {
		$length = strlen( $needle );
		if ( 0 === $length ) {
			return true;
		}

		return ( substr( $haystack, - $length ) === $needle );
	}

	/**
	 * Method get_nice_url()
	 *
	 * Grabs url.
	 *
	 * @param string $url Website URL.
	 * @param bool   $showHttp Show HTTP.
	 *
	 * @return string $url.
	 */
	public static function get_nice_url( $url, $showHttp = false ) {

		if ( self::starts_with( $url, 'http://' ) ) {
			if ( ! $showHttp ) {
				$url = substr( $url, 7 );
			}
		} elseif ( self::starts_with( $url, 'https://' ) ) {
			if ( ! $showHttp ) {
				$url = substr( $url, 8 );
			}
		} elseif ( $showHttp ) {
				$url = 'http://' . $url;
		}

		if ( self::ends_with( $url, '/' ) ) {
			if ( ! $showHttp ) {
				$url = substr( $url, 0, strlen( $url ) - 1 );
			}
		} else {
			$url = $url . '/';
		}

		return $url;
	}

	/**
	 * Debugging log info.
	 *
	 * Sets logging for debugging purpose
	 *
	 * @param string $message Log info message.
	 */
	public static function log_info( $message ) {
		$cron = '';
		if ( defined( 'DOING_CRON' ) && DOING_CRON ) {
			$cron = 'CRON :: ';
		}
		// Set color: 0 - LOG, 1 - WARNING, 2 - INFO, 3- DEBUG.
		$log_color = 2;
		do_action( 'mainwp_log_action', 'SSL Monitor:: ' . $cron . $message, MAINWP_SSL_MONITOR_LOG_PRIORITY, $log_color );
	}
}
