<?php

/**
 * MainWP SSL Monitor CLI
 *
 * This file handles CLI interactions.
 *
 * @package MainWP/Extensions
 */

namespace MainWP\Extensions\SSL_Monitor;

if ( ! defined( 'WP_CLI' ) ) {
	return;
}

/**
 * Class MainWP_SSL_Monitor_WP_CLI_Command
 */
class MainWP_SSL_Monitor_WP_CLI_Command extends \WP_CLI_Command {

	/**
	 * Checks if plugins are loaded and fire off init_wpcli_commands()
	 */
	public static function init() {
		add_action( 'plugins_loaded', array( self::class, 'init_wpcli_commands' ), 99999 );
	}

	/**
	 * Initiates MainWP WP-CLI commands.
	 */
	public static function init_wpcli_commands() {
		\WP_CLI::add_command( 'mainwp-ssl-monitor', self::class );
	}

	/**
	 * Run SSL Monitor check
	 *
	 * ## OPTIONS
	 *
	 * [<websiteid>]
	 * : The id of the child site to check.
	 *
	 * [--all]
	 * : If set, all child sites report will be show.
	 *
	 * ## EXAMPLES
	 *
	 *     wp mainwp-ssl-monitor check 2
	 *     wp mainwp-ssl-monitor check --all
	 *
	 * ## Synopsis [<websiteid>] [--all]
	 * ## EXAMPLES
	 *
	 * @param array $args       Scan arguments.
	 * @param array $assoc_args Associated arguments.
	 *
	 * @return void
	 */
	public function check( $args, $assoc_args ) {

		$sites = array();
		if ( count( $args ) > 0 ) {
			$args_exploded = explode( ',', $args[0] );
			foreach ( $args_exploded as $arg ) {
				if ( ! is_numeric( trim( $arg ) ) ) {
					\WP_CLI::error( 'Child site ids should be numeric.' );
				}

				$sites[] = trim( $arg );
			}
		}

		if ( ( count( $sites ) == 0 ) && ( ! isset( $assoc_args['all'] ) ) ) {
			\WP_CLI::error( 'Please specify one or more child sites, or use --all.' );
			return;
		}

		$all = false;
		if ( isset( $assoc_args['all'] ) ) {
			$all = true;
		}

		if ( $all ) {
			$websites = MainWP_SSL_Monitor_Utility::get_websites();
			if ( empty( $websites ) ) {
				\WP_CLI::error( __( 'Sites not found.', 'mainwp-ssl-monitor-extension' ) );
				return;
			}
			$id = '';
			foreach ( $websites as $website ) {
				$site_id  = $website['id'];
				$site_url = $website['url'];
				$ssl_site = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $site_id );
				$id       = isset( $ssl_site->id ) ? $ssl_site->id : 0;
				$result   = MainWP_SSL_Monitor_Utility::get_ssl_certificate_info( $site_url, $id, $site_id );

				if ( ! empty( $result ) ) {
					\WP_CLI::line( $result );
				} else {
					\WP_CLI::error( __( 'An undefined error occured.', 'mainwp-ssl-monitor-extension' ) );
				}
			}
			return;
		} else {
			$dbwebsites = MainWP_SSL_Monitor_Utility::get_db_sites( $sites );
			if ( empty( $dbwebsites ) ) {
				\WP_CLI::error( __( 'Sites not found.', 'mainwp-ssl-monitor-extension' ) );
				return;
			}
			$id = '';
			foreach ( $dbwebsites as $website ) {
				$site_id  = $website->id;
				$site_url = $website->url;
				$ssl_site = MainWP_SSL_Monitor_DB::get_instance()->get_ssl_monitor_by( 'site_id', $site_id );
				$id       = isset( $ssl_site->id ) ? $ssl_site->id : 0;
				$result   = MainWP_SSL_Monitor_Utility::get_ssl_certificate_info( $site_url, $id, $site_id );

				if ( ! empty( $result ) ) {
					\WP_CLI::line( $result );
				} else {
					\WP_CLI::error( __( 'An undefined error occured.', 'mainwp-ssl-monitor-extension' ) );
				}
			}
			return;
		}
	}

	/**
	 *
	 * View SSL Monitor data for Child Sites
	 *
	 * ## OPTIONS
	 *
	 * [<websiteid>]
	 * : The id (or ids, comma separated) of the child sites.
	 *
	 * [--all]
	 * : If set, all child sites report will be show.
	 *
	 * ## EXAMPLES
	 *
	 *     wp mainwp-ssl-monitor info 2,5
	 *     wp mainwp-ssl-monitor info --all
	 *
	 * ## Synopsis [<websiteid>] [--all]
	 *
	 * @param array $args       Scan arguments.
	 * @param array $assoc_args Associated arguments.
	 *
	 * @return void
	 */
	public function info( $args, $assoc_args ) {

		$sites = array();
		if ( count( $args ) > 0 ) {
			$args_exploded = explode( ',', $args[0] );
			foreach ( $args_exploded as $arg ) {
				if ( ! is_numeric( trim( $arg ) ) ) {
					\WP_CLI::error( 'Child site ids should be numeric.' );
				}

				$sites[] = trim( $arg );
			}
		}

		if ( ( count( $sites ) == 0 ) && ( ! isset( $assoc_args['all'] ) ) ) {
			\WP_CLI::error( 'Please specify one or more child sites, or use --all.' );
			return;
		}

		$all = false;
		if ( isset( $assoc_args['all'] ) ) {
			$all = true;
		}

		if ( $all ) {
			$websites = MainWP_SSL_Monitor_Utility::get_websites();
		} else {
			$dbwebsites = MainWP_SSL_Monitor_Utility::get_db_sites( $sites );
			$websites   = array();
			if ( is_array( $dbwebsites ) ) {
				foreach ( $dbwebsites as $site ) {
					$website    = array(
						'name' => $site->name,
						'url'  => $site->url,
						'id'   => $site->id,
					);
					$websites[] = $website;
				}
			}
		}
		if ( empty( $websites ) ) {
			\WP_CLI::error( __( 'Sites not found.', 'mainwp-ssl-monitor-extension' ) );
			return;
		}
		if ( is_array( $websites ) ) {
			foreach ( $websites as $website ) {
				MainWP_SSL_Monitor_Admin::handle_wp_cli_sites_ssl_info( $website );
			}
		}
	}
}
